﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ThirdPersonMovement : MonoBehaviour
{
    public CharacterController controller;
    public Transform cam;
    public SetControls setControls;

    //Player Speed
    public float speed = 6f;

    //Player Turn Speed/ Velocity
    public float turnSmoothTime = 0.1f;
    float turnSmoothVelocity;
    
    //Jump
    Vector3 velocity;
    public float gravity = -9.81f;
    public float jumpHeight = 3f;

    //Ground Check/ isGrounded
    public Transform groundCheck;
    public float groundDistance = 0.4f;
    public LayerMask groundMask;

    bool isGrounded;

    private void Start()
    {
        setControls.GetComponent<SetControls>();
    }

    // Update is called once per frame
    void Update()
    {
        setControls.SetOSControls();

        //Check for ground below player
        isGrounded = Physics.CheckSphere(groundCheck.position, groundDistance, groundMask);

        //reset velocity to -2 if grounded. Otherwise velocity will exponentially increase while grounded.
        if (isGrounded && velocity.y < 0)
        {
            velocity.y = -2f;
        }
        
        velocity.y -= gravity * Time.deltaTime;

        controller.Move(velocity * Time.deltaTime);

        //Player Movement
        float horizontal = Input.GetAxisRaw("Horizontal");
        float vertical = Input.GetAxisRaw("Vertical");
        Vector3 direction = new Vector3(horizontal, 0f, vertical).normalized;

        //Rotation + Move to Camera forward
        if(direction.magnitude >= 0.1f)
        {
            float targetAngle = Mathf.Atan2(direction.x, direction.z) * Mathf.Rad2Deg + cam.eulerAngles.y;
            float angle = Mathf.SmoothDampAngle(transform.eulerAngles.y, targetAngle, ref turnSmoothVelocity, turnSmoothTime);
            transform.rotation = Quaternion.Euler(0f, angle, 0f);

            Vector3 moveDir = Quaternion.Euler(0f, targetAngle, 0f) * Vector3.forward;
            controller.Move(moveDir.normalized * speed * Time.deltaTime);
        }

        //Jump

        if (Input.GetKeyDown(KeyCode.Space) && isGrounded || setControls.jump && isGrounded)
        {
            velocity.y = Mathf.Sqrt(jumpHeight);
        }

        //Fix Jump Edge Glitch
        if (isGrounded == false)
        {
            controller.stepOffset = 0f;
        }
        else
        {
            controller.stepOffset = 0.2f;
        }
    }
}
